//--------------------------------------------------------------
// File     : stm32_ub_ov9655.c
// Datum    : 17.12.2014
// Version  : 1.1
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : DCMI, DMA, STM32_UB_I2C1, STM32_UB_LCD_ST7783,
//            event. (UART)
// Funktion : Camera-Funktionen (Chip = OV9655)
// 
// Hinweis  : Betrieb per DMA und DCMI+I2C1-Schnittstelle
//
//              PA4  -> DCMI_HSYNC  = Camera HSYNC (HREF)
//              PA6  -> DCMI_PCLK   = Camera PCLK (PIXCLK)
//              PB6  -> DCMI_D5     = Camera D5
//              PB7  -> DCMI_VSYNC  = Camera VSYNC
//              PB8  -> I2C1-SCL    = Camera I2C-SCL
//              PB9  -> I2C1-SDA    = Camera I2C-SDA
//              PC6  -> DCMI_D0     = Camera D0
//              PC7  -> DCMI_D1     = Camera D1
//              PC8  -> DCMI_D2     = Camera D2
//              PC9  -> DCMI_D3     = Camera D3
//              PE4  -> DCMI_D4     = Camera D4
//              PE5  -> DCMI_D6     = Camera D6
//              PE6  -> DCMI_D7     = Camera D7  
//
//              DMA : entweder DMA2_STREAM1_CHANNEL1
//                    oder     DMA2_STREAM7_CHANNEL1
//
// Wichtig  : in der I2C-Library mssen die richtigen
//            CPU-Pins eingestellt sein (SCL=PB8, SDA=PB9)
//       
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_ov9655.h"


//--------------------------------------------------------------
// RAM Buffer fuer RAM-Snapshot
//--------------------------------------------------------------
#if USE_OV9655_RAM_SNAPSHOT==1
  #include "stm32_ub_uart.h"

  uint16_t ov9655_ram_buffer[OV9655_PIXEL];

  //--------------------------------------------------------------
  // Header fuer BMP-Transfer
  // (fix als einen kompletten Screen (320x240) im Landscape-Mode)
  //--------------------------------------------------------------
  #if OV9655_AKT_MODE==0
  const uint8_t BMP_HEADER_320x240[BMP_HEADER_LEN]={
  0x42,0x4D,0x36,0x84,0x03,0x00,0x00,0x00,0x00,0x00, // ID=BM, Filsize=(240x320x3+54)
  0x36,0x00,0x00,0x00,0x28,0x00,0x00,0x00,           // Offset=54d, Headerlen=40d
  0x40,0x01,0x00,0x00,0xF0,0x00,0x00,0x00,0x01,0x00, // W=320d, H=240d (landscape)
  0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x84,0x03,0x00, // 24bpp, unkomprimiert, Data=(240x320x3)
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,           // nc
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};          // nc
  #endif

  //--------------------------------------------------------------
 // Header fuer BMP-Transfer
  // (fix als einen kompletten Screen (160x120) im Landscape-Mode)
  //--------------------------------------------------------------
  #if OV9655_AKT_MODE==1
  const uint8_t BMP_HEADER_160x120[BMP_HEADER_LEN]={
  0x42,0x4D,0x36,0xE1,0x00,0x00,0x00,0x00,0x00,0x00, // ID=BM, Filsize=(120x160x3+54)
  0x36,0x00,0x00,0x00,0x28,0x00,0x00,0x00,           // Offset=54d, Headerlen=40d
  0xA0,0x00,0x00,0x00,0x78,0x00,0x00,0x00,0x01,0x00, // W=160d, H=120d (landscape)
  0x18,0x00,0x00,0x00,0x00,0x00,0x00,0xE1,0x00,0x00, // 24bpp, unkomprimiert, Data=(120x160x3)
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,           // nc
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};          // nc
  #endif

#endif



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
OV9655_ERR_t P_OV9655_CheckID(void);
OV9655_ERR_t P_OV9655_InitCamera(void);
void P_OV9655_InitIO(void);
void P_OV9655_InitDCMI(uint8_t mode);
void P_OV9655_InitDMA(uint8_t mode);
static unsigned dma_mode=0;


//--------------------------------------------------------------
// Init vom Camera-Modul
// Return_wert :
//      OV9655_OK , wenn OV9655 gefunden wurde
//   != OV9655_OK , fehler aufgetreten
//--------------------------------------------------------------
OV9655_ERR_t UB_OV9655_Init(void)
{
  OV9655_ERR_t ret_wert=OV9655_OK;
  ErrorStatus check;

  #if USE_OV9655_RAM_SNAPSHOT==1
  UB_Uart_Init();
  #endif

  // Init der I2C-Schnittstelle
  UB_I2C1_Init();

  // Init vom Display
  check=UB_LCD_Init();
  if(check==ERROR) {
    // Display nicht gefunden
    return(OV9655_LCD_ERR);
  }

  // check ob OV9655 vorhanden ist
  ret_wert = P_OV9655_CheckID();
  if(ret_wert==OV9655_OK) {
    // auf Landscape Mode umschalten
    UB_LCD_SetMode(LANDSCAPE);
    // Display lschen
    UB_LCD_FillScreen(RGB_COL_WHITE);
    // Camera initialsieren
    ret_wert = P_OV9655_InitCamera();
    if(ret_wert==OV9655_OK) {
      // init der IO-Pins
      P_OV9655_InitIO();
      // init vom DCMI
      P_OV9655_InitDCMI(0);
      // init vom DMA
      P_OV9655_InitDMA(0);
      dma_mode=0;
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// starten vom Capture-Mode
// es werden continuierlich Bilddaten auf dem Display angezeigt
//--------------------------------------------------------------
void UB_OV9655_CaptureStart(void)
{
  // Cursor auf Start setzen
  UB_LCD_SetCursor2Draw(LCD_WINDOW.xstart,LCD_WINDOW.ystart);
  // DMA-Mode einstellen
  if(dma_mode==1) {
    P_OV9655_InitDMA(0);
    dma_mode=0;
  }
  // DMA enable
  DMA_Cmd(OV9655_DCMI_DMA_STREAM, ENABLE);
  // DCMI init
  P_OV9655_InitDCMI(0);
  // DCMI enable
  DCMI_Cmd(ENABLE);
  // Capture enable
  DCMI_CaptureCmd(ENABLE);
}


//--------------------------------------------------------------
// stoppen vom Capture-Mode
// und stopen vom LCD zugriff
//--------------------------------------------------------------
void UB_OV9655_CaptureStop(void)
{
  // Capture disable
  DCMI_CaptureCmd(DISABLE);
  // DCMI disable
  DCMI_Cmd(DISABLE);
  // DMA disable
  DMA_Cmd(OV9655_DCMI_DMA_STREAM, DISABLE);
}


//--------------------------------------------------------------
// einen Snapshot machen
// (Capture-Mode muss AUS sein)
//--------------------------------------------------------------
void UB_OV9655_Snapshot(void)
{
  // Cursor auf Start setzen
  UB_LCD_SetCursor2Draw(LCD_WINDOW.xstart,LCD_WINDOW.ystart);
  // DMA-Mode einstellen
  if(dma_mode==1) {
    P_OV9655_InitDMA(0);
    dma_mode=0;
  }
  // DMA enable
  DMA_Cmd(OV9655_DCMI_DMA_STREAM, ENABLE);
  // DCMI init
  P_OV9655_InitDCMI(1);
  // DCMI enable
  DCMI_Cmd(ENABLE);
  // Capture enable
  DCMI_CaptureCmd(ENABLE);
}



//--------------------------------------------------------------
// Helligkeitswert einstellen
// Default = 0
// wert : 127 = maximal hell
//          0 = normal
//        255 = maximal dunkel
//--------------------------------------------------------------
void UB_OV9655_SetHelligkeit(uint8_t wert)
{
  UB_I2C1_WriteByte(OV9655_I2C_ADR, OV9655_I2C_REG_BRTN, wert);
}



//--------------------------------------------------------------
// einen Snapshot in einen RAM-Buffer machen
// (Capture-Mode muss AUS sein)
//--------------------------------------------------------------
#if USE_OV9655_RAM_SNAPSHOT==1
void UB_OV9655_Snapshot2RAM(void)
{
  // DMA-Mode einstellen
  if(dma_mode==0) {
    P_OV9655_InitDMA(1);
    dma_mode=1;
  }
  // DMA enable
  DMA_Cmd(OV9655_DCMI_DMA_STREAM, ENABLE);
  // DCMI init
  P_OV9655_InitDCMI(1);
  // DCMI enable
  DCMI_Cmd(ENABLE);
  // Capture enable
  DCMI_CaptureCmd(ENABLE);
}
#endif


//--------------------------------------------------------------
// den Snapshot vom RAM-Buffer ein ein BMP-File wandeln
// (kann dann z.B. per UART versendet werden
//--------------------------------------------------------------
#if USE_OV9655_RAM_SNAPSHOT==1
void UB_OV9655_RAM2BMP(void)
{
  uint32_t n,adr=0;
  uint16_t color,x,y;
  uint8_t r,g,b;

  // BMP-Header senden
  for(n=0;n<BMP_HEADER_LEN;n++) {
    #if OV9655_AKT_MODE==0
	  UB_Uart_SendByte(COM3,BMP_HEADER_320x240[n]);
    #else
      UB_Uart_SendByte(COM3,BMP_HEADER_160x120[n]);
    #endif
  }

  // Farb-Daten senden
  adr=0;
  for(x=0;x<OV9655_MAXX;x++) {
    for(y=0;y<OV9655_MAXY;y++) {
      color=ov9655_ram_buffer[adr];
      adr++;
      r=((color&0xF800)>>8);  // 5bit rot
      g=((color&0x07E0)>>3);  // 6bit gruen
      b=((color&0x001F)<<3);  // 5bit blau
      UB_Uart_SendByte(COM3,b);
      UB_Uart_SendByte(COM3,g);
      UB_Uart_SendByte(COM3,r);
    }
  }

}
#endif


//--------------------------------------------------------------
// interne Funktion
// check der OV9655-ID
//      OV9655_OK , wenn OV9655 gefunden wurde
//   != OV9655_OK , fehler aufgetreten
//--------------------------------------------------------------
OV9655_ERR_t P_OV9655_CheckID(void)
{
  OV9655_ERR_t ret_wert=OV9655_OK;
  int16_t int_wert;

  // Reset vom OV9655
  int_wert=UB_I2C1_WriteByte(OV9655_I2C_ADR, OV9655_I2C_REG_COM7, 0x80);
  if(int_wert<0) return(OV9655_I2C_ERR);

  // kleine Pause
  UB_I2C1_Delay(OV9655_I2C_RES_PAUSE);

  // Check auf ID-HIByte
  int_wert=UB_I2C1_ReadByte(OV9655_I2C_ADR, OV9655_I2C_REG_MIDH);
  if(int_wert<0) return(OV9655_I2C_ERR);
  if(int_wert!=0x7F) return(OV9655_MIDH_ERR);

  // Check auf ID-LoByte
  int_wert=UB_I2C1_ReadByte(OV9655_I2C_ADR, OV9655_I2C_REG_MIDL);
  if(int_wert<0) return(OV9655_I2C_ERR);
  if(int_wert!=0xA2) return(OV9655_MIDL_ERR);

  // Check auf Produkt-ID
  int_wert=UB_I2C1_ReadByte(OV9655_I2C_ADR, OV9655_I2C_REG_PID);
  if(int_wert<0) return(OV9655_I2C_ERR);
  if(int_wert!=0x96) return(OV9655_PID_ERR);

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// Init aller IO-Pins
//--------------------------------------------------------------
void P_OV9655_InitIO(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  //-----------------------------------------
  // Clock Enable von Port-A, Port-B, Port-C und Port-E
  //-----------------------------------------
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA,ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB,ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC,ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE,ENABLE);

  //-----------------------------------------
  // Alle Pins von Port-A initialisieren
  //-----------------------------------------
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource4, GPIO_AF_DCMI); // PA4=DCMI_HSYNC -> HSNYC
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_DCMI); // PA6=DCMI_PCLK  -> PCLK  
  
  // Structur fuer Port-A
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4 | GPIO_Pin_6;

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  // Config von Port-A
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  //-----------------------------------------
  // Alle Pins von Port-B initialisieren
  //-----------------------------------------
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_DCMI); // PB6=DCMI_D5    -> D5
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_DCMI); // PB7=DCMI_VSYNC -> VSYNC

  // Structur fuer Port-B
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  // Config von Port-B
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  //-----------------------------------------
  // Alle Pins von Port-C initialisieren
  //-----------------------------------------
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource6, GPIO_AF_DCMI); // PC6=DCMI_D0    -> D0
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource7, GPIO_AF_DCMI); // PC7=DCMI_D1    -> D1
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource8, GPIO_AF_DCMI); // PC8=DCMI_D2    -> D2
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource9, GPIO_AF_DCMI); // PC9=DCMI_D3    -> D3

  // Structur fuer Port-C
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  // Config von Port-C
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  //-----------------------------------------
  // Alle Pins von Port-E initialisieren
  //-----------------------------------------
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource4, GPIO_AF_DCMI); // PE4=DCMI_D4    -> D4
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource5, GPIO_AF_DCMI); // PE5=DCMI_D6    -> D6
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource6, GPIO_AF_DCMI); // PE6=DCMI_D7    -> D7

  // Structur fuer Port-E
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6;

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  // Config von Port-E
  GPIO_Init(GPIOE, &GPIO_InitStructure);

}


//--------------------------------------------------------------
// interne Funktion
// Init vom DCMI
// mode 0 = continuous
//      1 = snapshot
//--------------------------------------------------------------
void P_OV9655_InitDCMI(uint8_t mode)
{
  DCMI_InitTypeDef DCMI_InitStructure;

  // Clock enable
  RCC_AHB2PeriphClockCmd(RCC_AHB2Periph_DCMI, ENABLE);

  // DCMI init
  if(mode==0) {
    DCMI_InitStructure.DCMI_CaptureMode = DCMI_CaptureMode_Continuous;
  }
  else {
    DCMI_InitStructure.DCMI_CaptureMode = DCMI_CaptureMode_SnapShot;
  }
  DCMI_InitStructure.DCMI_SynchroMode = DCMI_SynchroMode_Hardware;
  DCMI_InitStructure.DCMI_PCKPolarity = DCMI_PCKPolarity_Falling;
  DCMI_InitStructure.DCMI_VSPolarity = DCMI_VSPolarity_High;
  DCMI_InitStructure.DCMI_HSPolarity = DCMI_HSPolarity_High;
  DCMI_InitStructure.DCMI_CaptureRate = DCMI_CaptureRate_All_Frame;
  DCMI_InitStructure.DCMI_ExtendedDataMode = DCMI_ExtendedDataMode_8b; 
  DCMI_Init(&DCMI_InitStructure);
}


//--------------------------------------------------------------
// interne Funktion
// Init vom DMA2
// mode 0 = LCD-Display
//      1 = RAM-BUffer
//--------------------------------------------------------------
void P_OV9655_InitDMA(uint8_t mode)
{
  DMA_InitTypeDef  DMA_InitStructure;

  // Clock enable
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2, ENABLE);

  // DMA deinit
  DMA_Cmd(OV9655_DCMI_DMA_STREAM, DISABLE);
  DMA_DeInit(OV9655_DCMI_DMA_STREAM);

  // DMA init
  if(mode==0) {
    DMA_InitStructure.DMA_Channel = OV9655_DCMI_DMA_CHANNEL;
    DMA_InitStructure.DMA_PeripheralBaseAddr = OV9655_DCMI_REG_DR_ADDRESS;
    DMA_InitStructure.DMA_Memory0BaseAddr = OV9655_DCMI_MEMORY_ADR;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralToMemory;
    DMA_InitStructure.DMA_BufferSize = 1;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Disable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Word;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
    DMA_InitStructure.DMA_Priority = DMA_Priority_High;
    DMA_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Enable;
    DMA_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_Full;
    DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
    DMA_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;
    DMA_Init(OV9655_DCMI_DMA_STREAM, &DMA_InitStructure);
  }
  else {
    #if USE_OV9655_RAM_SNAPSHOT==1
    DMA_InitStructure.DMA_Channel = OV9655_DCMI_DMA_CHANNEL;
    DMA_InitStructure.DMA_PeripheralBaseAddr = OV9655_DCMI_REG_DR_ADDRESS;
    DMA_InitStructure.DMA_Memory0BaseAddr = (uint32_t)&ov9655_ram_buffer;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralToMemory;
    DMA_InitStructure.DMA_BufferSize = OV9655_PIXEL;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Word;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
    DMA_InitStructure.DMA_Priority = DMA_Priority_High;
    DMA_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Enable;
    DMA_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_Full;
    DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
    DMA_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;
    DMA_Init(OV9655_DCMI_DMA_STREAM, &DMA_InitStructure);
    #endif
  }
}


//--------------------------------------------------------------
// Register init Werte fr QVGA-Mode
// QVGA = 320x240 Pixel
//--------------------------------------------------------------
#if OV9655_AKT_MODE == 0
static uint8_t OV9655_QVGA_TAB[][2]=
{
  {0x00,0x00},{0x01,0x80},{0x02,0x80},{0x03,0x02},{0x04,0x03},{0x09,0x01},
  {0x0b,0x57},{0x0e,0x61},{0x0f,0x40},{0x11,0x01},{0x12,0x62},{0x13,0xc7},
  {0x14,0x3a},{0x16,0x24},{0x17,0x18},{0x18,0x04},{0x19,0x01},{0x1a,0x81},
  {0x1e,0x00},{0x24,0x3c},{0x25,0x36},{0x26,0x72},{0x27,0x08},{0x28,0x08},
  {0x29,0x15},{0x2a,0x00},{0x2b,0x00},{0x2c,0x08},{0x32,0x12},{0x33,0x00},
  {0x34,0x3f},{0x35,0x00},{0x36,0x3a},{0x38,0x72},{0x39,0x57},{0x3a,0xcc},
  {0x3b,0x04},{0x3d,0x99},{0x3e,0x02},{0x3f,0xc1},{0x40,0xc0},{0x41,0x41},
  {0x42,0xc0},{0x43,0x0a},{0x44,0xf0},{0x45,0x46},{0x46,0x62},{0x47,0x2a},
  {0x48,0x3c},{0x4a,0xfc},{0x4b,0xfc},{0x4c,0x7f},{0x4d,0x7f},{0x4e,0x7f},
  {0x4f,0x98},{0x50,0x98},{0x51,0x00},{0x52,0x28},{0x53,0x70},{0x54,0x98},
  {0x58,0x1a},{0x59,0x85},{0x5a,0xa9},{0x5b,0x64},{0x5c,0x84},{0x5d,0x53},
  {0x5e,0x0e},{0x5f,0xf0},{0x60,0xf0},{0x61,0xf0},{0x62,0x00},{0x63,0x00},
  {0x64,0x02},{0x65,0x20},{0x66,0x00},{0x69,0x0a},{0x6b,0x5a},{0x6c,0x04},
  {0x6d,0x55},{0x6e,0x00},{0x6f,0x9d},{0x70,0x21},{0x71,0x78},{0x72,0x11},
  {0x73,0x01},{0x74,0x10},{0x75,0x10},{0x76,0x01},{0x77,0x02},{0x7A,0x12},
  {0x7B,0x08},{0x7C,0x16},{0x7D,0x30},{0x7E,0x5e},{0x7F,0x72},{0x80,0x82},
  {0x81,0x8e},{0x82,0x9a},{0x83,0xa4},{0x84,0xac},{0x85,0xb8},{0x86,0xc3},
  {0x87,0xd6},{0x88,0xe6},{0x89,0xf2},{0x8a,0x24},{0x8c,0x80},{0x90,0x7d},
  {0x91,0x7b},{0x9d,0x02},{0x9e,0x02},{0x9f,0x7a},{0xa0,0x79},{0xa1,0x40},
  {0xa4,0x50},{0xa5,0x68},{0xa6,0x4a},{0xa8,0xc1},{0xa9,0xef},{0xaa,0x92},
  {0xab,0x04},{0xac,0x80},{0xad,0x80},{0xae,0x80},{0xaf,0x80},{0xb2,0xf2},
  {0xb3,0x20},{0xb4,0x20},{0xb5,0x00},{0xb6,0xaf},{0xb6,0xaf},{0xbb,0xae},
  {0xbc,0x7f},{0xbd,0x7f},{0xbe,0x7f},{0xbf,0x7f},{0xbf,0x7f},{0xc0,0xaa},
  {0xc1,0xc0},{0xc2,0x01},{0xc3,0x4e},{0xc6,0x05},{0xc7,0x81},{0xc9,0xe0},
  {0xca,0xe8},{0xcb,0xf0},{0xcc,0xd8},{0xcd,0x93},{0x12,0x63},{0x40,0x10},
  {0x15,0x08}
};
#endif

//--------------------------------------------------------------
// Register init Werte fr QQVGA-Mode
// QQVGA = 160x120 Pixel
//--------------------------------------------------------------
#if OV9655_AKT_MODE == 1
static uint8_t OV9655_QQVGA_TAB[][2]=
{
  {0x00,0x00},{0x01,0x80},{0x02,0x80},{0x03,0x02},{0x04,0x03},{0x09,0x01},
  {0x0b,0x57},{0x0e,0x61},{0x0f,0x40},{0x11,0x01},{0x12,0x62},{0x13,0xc7},
  {0x14,0x3a},{0x16,0x24},{0x17,0x18},{0x18,0x04},{0x19,0x01},{0x1a,0x81},
  {0x1e,0x00},{0x24,0x3c},{0x25,0x36},{0x26,0x72},{0x27,0x08},{0x28,0x08},
  {0x29,0x15},{0x2a,0x00},{0x2b,0x00},{0x2c,0x08},{0x32,0xa4},{0x33,0x00},
  {0x34,0x3f},{0x35,0x00},{0x36,0x3a},{0x38,0x72},{0x39,0x57},{0x3a,0xcc},
  {0x3b,0x04},{0x3d,0x99},{0x3e,0x0e},{0x3f,0xc1},{0x40,0xc0},{0x41,0x41},
  {0x42,0xc0},{0x43,0x0a},{0x44,0xf0},{0x45,0x46},{0x46,0x62},{0x47,0x2a},
  {0x48,0x3c},{0x4a,0xfc},{0x4b,0xfc},{0x4c,0x7f},{0x4d,0x7f},{0x4e,0x7f},
  {0x4f,0x98},{0x50,0x98},{0x51,0x00},{0x52,0x28},{0x53,0x70},{0x54,0x98},
  {0x58,0x1a},{0x59,0x85},{0x5a,0xa9},{0x5b,0x64},{0x5c,0x84},{0x5d,0x53},
  {0x5e,0x0e},{0x5f,0xf0},{0x60,0xf0},{0x61,0xf0},{0x62,0x00},{0x63,0x00},
  {0x64,0x02},{0x65,0x20},{0x66,0x00},{0x69,0x0a},{0x6b,0x5a},{0x6c,0x04},
  {0x6d,0x55},{0x6e,0x00},{0x6f,0x9d},{0x70,0x21},{0x71,0x78},{0x72,0x22},
  {0x73,0x02},{0x74,0x10},{0x75,0x10},{0x76,0x01},{0x77,0x02},{0x7A,0x12},
  {0x7B,0x08},{0x7C,0x16},{0x7D,0x30},{0x7E,0x5e},{0x7F,0x72},{0x80,0x82},
  {0x81,0x8e},{0x82,0x9a},{0x83,0xa4},{0x84,0xac},{0x85,0xb8},{0x86,0xc3},
  {0x87,0xd6},{0x88,0xe6},{0x89,0xf2},{0x8a,0x24},{0x8c,0x80},{0x90,0x7d},
  {0x91,0x7b},{0x9d,0x02},{0x9e,0x02},{0x9f,0x7a},{0xa0,0x79},{0xa1,0x40},
  {0xa4,0x50},{0xa5,0x68},{0xa6,0x4a},{0xa8,0xc1},{0xa9,0xef},{0xaa,0x92},
  {0xab,0x04},{0xac,0x80},{0xad,0x80},{0xae,0x80},{0xaf,0x80},{0xb2,0xf2},
  {0xb3,0x20},{0xb4,0x20},{0xb5,0x00},{0xb6,0xaf},{0xb6,0xaf},{0xbb,0xae},
  {0xbc,0x7f},{0xbd,0x7f},{0xbe,0x7f},{0xbf,0x7f},{0xbf,0x7f},{0xc0,0xaa},
  {0xc1,0xc0},{0xc2,0x01},{0xc3,0x4e},{0xc6,0x05},{0xc7,0x82},{0xc9,0xe0},
  {0xca,0xe8},{0xcb,0xf0},{0xcc,0xd8},{0xcd,0x93},{0x12,0x63},{0x40,0x10},
  {0x15,0x08}
};
#endif

//--------------------------------------------------------------
// interne Funktion
// Init der OV9655 Camera im QVGA oder QVGA-Mode
// und einstellen der Bildgre und Position am Display
//      OV9655_OK , wenn kein Fehler
//   != OV9655_OK , fehler aufgetreten
//--------------------------------------------------------------
OV9655_ERR_t P_OV9655_InitCamera(void)
{
  OV9655_ERR_t ret_wert=OV9655_OK;
  int16_t int_wert;
  uint32_t i,anzahl;

  #if OV9655_AKT_MODE==0
    // QVGA = 320x240 Pixel
    anzahl=sizeof(OV9655_QVGA_TAB)/2;
    for(i=0; i<anzahl; i++) {
      int_wert=UB_I2C1_WriteByte(OV9655_I2C_ADR, OV9655_QVGA_TAB[i][0], OV9655_QVGA_TAB[i][1]);
      UB_I2C1_Delay(OV9655_I2C_WR_PAUSE);
      if(int_wert<0) {
        ret_wert=OV9655_I2C_ERR;
        break;
      }
    }
    // Displaysize = 320x240, Vollbild
    UB_LCD_SetWindow(0,0,239,319);
    UB_LCD_SetCursor2Draw(LCD_WINDOW.xstart,LCD_WINDOW.ystart);
  #else
    // QQVGA = 160x120 Pixel
    anzahl=sizeof(OV9655_QQVGA_TAB)/2;
    for(i=0; i<anzahl; i++) {
      int_wert=UB_I2C1_WriteByte(OV9655_I2C_ADR, OV9655_QQVGA_TAB[i][0], OV9655_QQVGA_TAB[i][1]);
      UB_I2C1_Delay(OV9655_I2C_WR_PAUSE);
      if(int_wert<0) {
       ret_wert=OV9655_I2C_ERR;
       break;
      }
    }
    // Displaysize = 160x120, Center
    UB_LCD_SetWindow(60,80,179,239);
    UB_LCD_SetCursor2Draw(LCD_WINDOW.xstart,LCD_WINDOW.ystart);
  #endif

  return(ret_wert);
}



